@@ -121,6 +121,8 @@ def show_message_box(stdscr, message, wait_key=True):
     if message is None:
         return
     lines = [line.strip() for line in message.strip().split('\n')]
+    if wait_key:
+        lines.append("press-a-key")
     h, w = stdscr.getmaxyx()
     #
     box_h = 8
@@ -199,8 +201,14 @@ def on_backup(stdscr):
 #
 # --------------------------------------------------------------------------------
 
-def on_copy(name, dbase, other_dbase, stdscr, wordcopy="COPY", asking=True):
-    logging.debug(f"copying - {name}/{dbase} -> {other_dbase};")
+def on_copy(name, dbase, other_dbase, stdscr, wordcopy="COPY", asking=True,
+            silent=True,
+            new_measurement=None
+            ):
+    """
+    I try to have this as the only interface to all copy_measurement calls. This means +silent+new_measurement
+    """
+    logging.debug(f"copying - {name} from {dbase} -> db:{other_dbase};")
     if stdscr:
         confirm = True
         if asking:
@@ -209,16 +217,21 @@ def on_copy(name, dbase, other_dbase, stdscr, wordcopy="COPY", asking=True):
             if (name is None) or (dbase is None) or (other_dbase is None):
                 pass
             else:
-                show_message_box(stdscr, "... preforming copy ...")
-                res = maninf.copy_measurement(name, dbase, other_dbase, silent=True)
+                show_message_box(stdscr, f"... performing {wordcopy} ...", wait_key=False)
+                #
+                res = maninf.copy_measurement2(name, dbase, other_dbase, silent=silent, new_measurement=new_measurement)
+                #res = maninf.copy_measurement(name, dbase, other_dbase, silent=silent, new_measurement=new_measurement)
+                ##### res = maninf.copy_measurement(name, dbase, dbase, silent=True, new_measurement_name=new_name) # rename part
+                #
+                show_message_box(stdscr, f"... {wordcopy} done ...", wait_key=False)
                 logging.debug( f"{res}")
-            logging.debug(f"copied '{name}'")
+            logging.debug(f"{wordcopy}-ed '{name}'")
         else:
-            logging.debug(f"copying cancelled for '{name}'")
+            logging.debug(f"{wordcopy}-ing cancelled for '{name}'")
     else:
         # If no stdscr provided, assume yes or handle differently
         # maninf.drop_database(name)
-        logging.debug(f"copied '{name}' without confirmation")
+        logging.debug(f"{wordcopy}-ed '{name}' without confirmation")
     pass
 
 # ================================================================================
@@ -241,7 +254,7 @@ def on_delete_database(name, stdscr):
     if stdscr:
         confirm = confirm_dialog(stdscr, f"Delete database '{name}'?")
         if confirm:
-            show_message_box(stdscr, "... preforming drop db ...")
+            show_message_box(stdscr, "... preforming drop db ...", wait_key=False)
             maninf.drop_database(name)
             logging.debug(f"Deleted '{name}'")
         else:
@@ -262,7 +275,7 @@ def on_delete_measurement(name, database, stdscr, asking=True):
         if asking:
             confirm = confirm_dialog(stdscr, f"Delete measurement '{name}' of {database}?")
         if confirm:
-            show_message_box("... preforming delete ...")
+            show_message_box(stdscr, "... preforming delete ...", wait_key=False)
             maninf.delete_measurement(name, database)
             logging.debug(f"Deleted measurement '{name}' from {database}")
         else:
@@ -353,15 +366,21 @@ def input_dialog(stdscr, prompt):
 # ================================================================================
 #    MAIN *****************************
 # --------------------------------------------------------------------------------
+#    MAIN *****************************
+# --------------------------------------------------------------------------------
+#    MAIN *****************************
+# --------------------------------------------------------------------------------
 
 @click.command()
 @click.option('--logfile', default='/tmp/mcinflux_debug.log', help='Log file path')
 @click.option('--admin', "-a", is_flag=True, help='Act as admin. Else user privileges are used')
 @click.option('--test_only', "-t", is_flag=True, help='Test, do not run the commander')
-def main(logfile, admin, test_only):
+@click.option('--skip_test', "-s", is_flag=True, help='Skip all testing')
+def main(logfile, admin, test_only, skip_test):
 
     global user_glob_admin
     global left_disk_mode, right_disk_mode
+    maninf.LOGFILENAME = logfile
 
     # check PyPI for version
     is_there_new_version(package="jusfltuls", printit=True, printall=True)
@@ -388,6 +407,7 @@ def main(logfile, admin, test_only):
     ok = maninf.verify_files()
     if not ok:
         print("X...  files missing")
+        # CREATE FILES AND MKDIR !!!!
         sys.exit(1)
 
 
@@ -409,7 +429,10 @@ def main(logfile, admin, test_only):
         print(f"{fg.red} FAILED {fg.default} ")
         print(f"X... {fg.red}INFLUX SEEMS TO HAVE  NO USER AND PASSWORD ... {fg.default}")
         print(HELP_NEWDB)
-        sys.exit(1)
+        if not skip_test:
+            sys.exit(1)
+        else:
+            print(".... skipping exit ", end="")
     print(f"{fg.green} OK {fg.default} ")  # pentesting OK
 
     print(f"i... user: {fg.green}{u}{fg.default} on machine: {fg.yellow}{m}{fg.default} , I am going IN")  # ******************<<<<<
@@ -421,7 +444,7 @@ def main(logfile, admin, test_only):
     # I THINK I AM IN ************************** ****************************************************************************
 
     if len(res) == 0:
-        print(f"X... {fg.red} NO DATABASES seen !!! {fg.default}   .........   THOROUGH TESTING STARTS .....")
+        print(f"X... {fg.red} NO DATABASES seen !! {fg.default}   .........   THOROUGH TESTING STARTS .....")
 
 
         # ADMIN ONLY ********************************************
@@ -481,7 +504,10 @@ def main(logfile, admin, test_only):
                         print(f"X... {fg.red} '{i}' user is ADMIN {fg.default}")
             #---------------------------------- res2 ----
             print(f"i... end of the abnormal testing")
-            sys.exit(1) # --------------------------------------------- ABNORMAL TESTING -----------------------------------------
+            if not skip_test:
+                sys.exit(1) # --------------------------------------------- ABNORMAL TESTING -----------------------------------------
+            else:
+                print(".... skipping exit ", end="")
 
 
         print(f"{fg.green} OK {fg.default} ")  # pentesting OK
@@ -500,10 +526,17 @@ def main(logfile, admin, test_only):
 
 
         # -------------------------------------------------------- FORCE QUIT WHEN USER SWITCH
-        sys.exit(0)
+        if not skip_test:
+            sys.exit(1) # --------------------------------------------- ABNORMAL TESTING -----------------------------------------
+        else:
+            print(".... skipping exit ", end="")
+        #sys.exit(0)
 
     print("i... initial sequence finished.")
     if test_only:
+        print("""
+        sudo influxd restore -portable -host 127.0.0.1:8088 /root/influx-backup` WORKED when db 1.8.4 is live!
+        """)
         sys.exit(0)
 
     def curses_main(stdscr):
@@ -665,7 +698,7 @@ def main(logfile, admin, test_only):
                         dbase = right_db_selected
                         other_dbase = left_db_selected
 
-                    res = maninf.export_measurement(name, dbase, silent=True) # This is measurement to csv
+                    res = maninf.export_measurement(name, dbase, silent=True) # This is measurement to csv ORIGINAL WAY !
                     logging.debug(f"{res}")
                     #show_message_box(stdscr, f"{res}")
                 refresh_panels()
@@ -857,7 +890,7 @@ def main(logfile, admin, test_only):
                         dbase = right_db_selected
                         other_dbase = left_db_selected
                     logging.debug(f"copy no disk {name} from {dbase} to {other_dbase} ")
-                    on_copy(name, dbase, other_dbase, stdscr)
+                    on_copy(name, dbase, other_dbase, stdscr)  # *********************************     '5' COPY FROM DBASE TO DBASE ******
                     refresh_panels()
                 else:# ------------------- one panel is disk
                     if active_panel == 'left' and right_disk_mode: # COPY TO DISK L- R ----------------------------------
@@ -866,10 +899,12 @@ def main(logfile, admin, test_only):
                         if name and dbase: # copy measurement
                             #filename = input_dialog(stdscr, f"Export measurement '{name}' from {dbase} to file (in mirror folder): ")
                             filename = f"{name}.csv"
-                            logging.debug(f"copy to disk: {name} from {dbase} to {filename} ")
+                            logging.debug(f"copy to disk left2right: {name} from {dbase} to {filename} ")
                             if filename:
                                 filepath = os.path.join(DISK_MIRROR_PATH, filename)
-                                res = maninf.export_measurement(name, dbase, filepath=filepath) # csv export
+                                show_message_box(stdscr, f"... export2disk  ...", wait_key=False)
+                                res = maninf.export_measurement2(name, dbase, filepath=filepath) # csv export
+                                refresh_panels()
                                 #show_message_box(stdscr, res)
                         elif name and dbase is None: # BACKUP
                             filename = f"{name}.exported"
@@ -878,7 +913,9 @@ def main(logfile, admin, test_only):
                                 show_message_box(stdscr, f"NO BACKUP TO EXISTING BACKUP \n{filepath}")
                             else:
                                 logging.debug(f"copy DB to disk: {name}  to {filepath} ")
+                                show_message_box(stdscr, f"... backup portable  ...", wait_key=False)
                                 maninf.backup_portable(filepath, name ) # ONE  DB BACKUP
+                                refresh_panels()
 
                     elif  active_panel == 'right' and left_disk_mode: # COPY TO DISK R - L -----------------------------------------
                         name = right_items[right_idx]
@@ -886,9 +923,12 @@ def main(logfile, admin, test_only):
                         if name and dbase: # copy measurement
                             #filename = input_dialog(stdscr, f"Export measurement '{name}' from {dbase} to file (in mirror folder): ")
                             filename = f"{name}.csv"
+                            logging.debug(f"copy to disk right2left: {name} from {dbase} to {filename} ")
                             if filename:
                                 filepath = os.path.join(DISK_MIRROR_PATH, filename)
-                                res = maninf.export_measurement(name, dbase, filepath=filepath) # csv export
+                                show_message_box(stdscr, f"... export2disk  ...", wait_key=False)
+                                res = maninf.export_measurement2(name, dbase, filepath=filepath) # csv export
+                                refresh_panels()
                                 #show_message_box(stdscr, res)
                         elif name and dbase is None: # BACKUP
                             filename = f"{name}.exported"
@@ -899,7 +939,7 @@ def main(logfile, admin, test_only):
                                 logging.debug(f"copy DB to disk: {name}  to {filepath} ")
                                 maninf.backup_portable(filepath, name ) # ONE  DB BACKUP
 
-                    elif active_panel == "right" and right_disk_mode: # RESTORE TO  INFLUX
+                    elif active_panel == "right" and right_disk_mode: # * * * * * * *  RESTORE TO  INFLUX
                         #show_message_box(stdscr, "FEATURE NOT IMPLEMENTED\n copying disk to DB right side")
                         name = right_items[right_idx]
                         if name.find(".exported") > 0:
@@ -911,7 +951,7 @@ def main(logfile, admin, test_only):
                         else:
                             show_message_box(stdscr, "FEATURE NOT IMPLEMENTED\n copying disk to infl right side")
                         pass
-                    elif active_panel == "left" and left_disk_mode:  # RESTORE TO  INFLUX
+                    elif active_panel == "left" and left_disk_mode:  # * * * * * * * * RESTORE TO  INFLUX
                         name = left_items[left_idx]
                         if name.find(".exported") > 0:
                             basename = name.split(".exported")[0]
@@ -966,8 +1006,9 @@ def main(logfile, admin, test_only):
                         else:logging.debug("empty string for newname")
                     else: # COPY AND DELETE
                         logging.debug(f"I am in measurement view and F6 mea=={name}")
-                        on_copy(name, dbase, other_dbase, stdscr, wordcopy="MOVE")
-                        on_delete_measurement(name, dbase, stdscr, asking=False) # in move
+                        on_copy(name, dbase, other_dbase, stdscr, wordcopy="MOVE")  # ********************* '6' COPY PART .....
+                        refresh_panels() # prepare space for next dialog
+                        on_delete_measurement(name, dbase, stdscr, asking=False) # ************************* '6' DELETE PART
                 refresh_panels()
                 #on_move()
                 #refresh_panels()
@@ -999,28 +1040,33 @@ def main(logfile, admin, test_only):
                                 except Exception as e:
                                     show_message_box(stdscr, f"Rename failed: {e}")
                                 refresh_panels()
-                    else:# ----------------------------------------  part -----------------
+                    else:# ---------------normal influxdb mode---------------
                         # Rename measurement in influxdb
                         name = left_items[left_idx]
                         dbase = left_db_selected
-                        logging.debug(f" key {chr(key)} pressed;  infl rename L; {name} {dbase}   ")
-                        if name and dbase:
+                        logging.debug(f" key {chr(key)} pressed;  infl rename L; {name} from {dbase}   ")
+                        if name and dbase: # measurement from database
                             new_name = input_dialog(stdscr, f"Rename measurement '{name}' to: ")
                             if new_name and new_name != name:
                                 logging.debug(f" key {chr(key)} pressed;  infl rename; old={name} db={dbase}   ")
                                 logging.debug(f" key {chr(key)} pressed;  infl rename; new={new_name} db={dbase}   ")
                                 # Copy measurement to new_name then delete old
-                                res_copy = maninf.copy_measurement(name, dbase, dbase, silent=True, new_measurement_name=new_name)
-                                logging.debug(f" key {chr(key)} pressed;  infl rename; result: /{res_copy}/  ")
-                                #
-                                #  NOT DELETING HERE TO BE SURE
-                                #if "error" not in res_copy.lower():
-                                    #maninf.delete_measurement(name, dbase)
-                                    ## #show_message_box(stdscr, f"Renamed measurement '{name}' to '{new_name}'")
-                                #else:
-                                #    show_message_box(stdscr, f"Rename failed: {res_copy}")
-                                refresh_panels()
-                else:# right panel is active ------------------------------------------
+                                #on_copy
+                                on_copy(name, dbase, dbase, stdscr, silent=True, new_measurement=new_name)  # ****************    '^' rename in same DBASE ***
+                        elif dbase is None:
+                            new_name = input_dialog(stdscr, f"Rename DB '{name}' to: ")
+                            if len(new_name) > 0:
+                                logging.debug(f"renaming database {name} to {new_name}")
+                                maninf.create_database(new_name) # same as '6'
+                                res = maninf.show_measurements(name)
+                                for i in res:
+                                    logging.debug(f" - copying {i} from {name} to {new_name}")
+                                    on_copy(i, name, new_name, stdscr, asking=False)
+                                    logging.debug(f" - not dropping {i} in {name}; ")
+                                name = input_dialog(stdscr, f"You drop {name} database yourself ")
+                            else:logging.debug("empty string for newname")
+
+                else:# ---------- right panel is active ------------------------------------------
                     if right_disk_mode:
                         selected_item = right_items[right_idx]
                         if selected_item == '..':
@@ -1038,7 +1084,7 @@ def main(logfile, admin, test_only):
                                     #show_message_box(stdscr, f"Renamed '{selected_item}' to '{new_name}'")
                                 except Exception as e:
                                     show_message_box(stdscr, f"Rename failed: {e}")
-                                refresh_panels()
+                                #refresh_panels()
                     else:# ---- influxdb mode ---------------
                         name = right_items[right_idx]
                         dbase = right_db_selected
@@ -1048,14 +1094,29 @@ def main(logfile, admin, test_only):
                             if new_name and new_name != name:
                                 logging.debug(f" key {chr(key)} pressed;  infl rename; old={name} db={dbase}   ")
                                 logging.debug(f" key {chr(key)} pressed;  infl rename; new={new_name} db={dbase}   ")
-                                res_copy = maninf.copy_measurement(name, dbase, dbase, silent=True, new_measurement_name=new_name)
-                                logging.debug(f" key {chr(key)} pressed;  infl rename; result: /{res_copy}/  ")
+                                #on_copy
+                                on_copy(name, dbase, dbase, stdscr, silent=True, new_measurement=new_name)  # ****************    '^' rename in same DBASE ****
+                        elif dbase is None:
+                            new_name = input_dialog(stdscr, f"Rename DB '{name}' to: ")
+                            if len(new_name) > 0:
+                                logging.debug(f"renaming database {name} to {new_name}")
+                                maninf.create_database(new_name) # same as '6'
+                                res = maninf.show_measurements(name)
+                                for i in res:
+                                    logging.debug(f" - copying {i} from {name} to {new_name}")
+                                    on_copy(i, name, new_name, stdscr, asking=False)
+                                    logging.debug(f" - not dropping {i} in {name}; ")
+                                name = input_dialog(stdscr, f"You drop {name} database yourself ")
+                            else:logging.debug("empty string for newname")
+
+                                #res_copy = maninf.copy_measurement(name, dbase, dbase, silent=True, new_measurement_name=new_name)
+                                #logging.debug(f" key {chr(key)} pressed;  infl rename; result: /{res_copy}/  ")
                                 #if "error" not in res_copy.lower():
                                 #    maninf.delete_measurement(name, dbase)
                                 #    #show_message_box(stdscr, f"Renamed measurement '{name}' to '{new_name}'")
                                 #else:
                                 #    show_message_box(stdscr, f"Rename failed: {res_copy}")
-                                refresh_panels()
+                refresh_panels()
 
             # ================================================================================
             #
@@ -1100,7 +1161,7 @@ def main(logfile, admin, test_only):
                                 on_delete_measurement(selected_item, left_db_selected, stdscr)
                     if active_panel == 'right':
                         if right_items:# right panel  ***********************
-                            logging.debug("8... deleting...right ... TO DO !!!")
+                            logging.debug("8... deleting...right ... TO DO !!")
                             selected_item = right_items[right_idx]
                             logging.debug(f"8. deleting...selected: {selected_item}, db={right_db_selected}")
                             if (selected_item != '..') and (right_db_selected is None): #  view ??? meas?
@@ -1289,4 +1350,5 @@ def main(logfile, admin, test_only):
     curses.wrapper(curses_main)
 
 if __name__ == "__main__":
+    #   LOGLIFE DEFINED IN click MAIN....
     main()
