@@ -10,10 +10,14 @@ from configparser import ConfigParser
 import socket
 import glob
 import json
+import sys
+import csv
+from influxdb import InfluxDBClient
+import logging # Should be initialized in main part in MAIN() !?!?!
 
 user_glob_admin = False
 
-LOGFILENAME = "/tmp/mcinflux_backup.log"
+LOGFILENAME = "/tmp/mcinflux_debug.log" # WILL BE REWRITTEN IN MAIN()
 
 
 # ================================================================================
@@ -40,7 +44,27 @@ def is_int(n):
 
 def verify_files( justprint=False):
     ok = True
-    FIS = ["~/.config/influxdb/totalconfig.conf"]
+    FIS = ["~/.config/influxdb/totalconfig.conf"]  # JUST ONE HERE!!
+    FIF = ["~/.config/influxdb"]
+    p = os.path.expanduser(FIF[0])
+    try:
+        if os.path.exists(p):
+            if os.path.isdir(p):
+                pass
+                #click.echo(f"Exists: {p}")
+            else:
+                printf(f"Error: path exists and is not a directory: {p}", err=True)
+                sys.exit(1)
+        else:
+            os.makedirs(p, exist_ok=True)
+            printf(f"!... Created FOLDER: {p}")
+    except PermissionError:
+        click.echo(f"Error: permission denied creating {p}", err=True)
+        sys.exit(1)
+    except Exception as e:
+        click.echo(f"Error: {e}", err=True)
+        sys.exit(1)
+    # Folder exists now
     for i in FIS:
         if justprint:
             print( i)
@@ -49,6 +73,18 @@ def verify_files( justprint=False):
             if not os.path.exists(FF):
                 ok = False
                 print(f"X... {FF} doesnt exist")
+                print(f"X... {FF} doesnt exist ... CREATING BASIC STRUCTURE !!!")
+                with open(FF, "a") as f:
+                    f.write("""
+# uvx crudini --get ~/.config/influxdb/totalconfig.conf  "influx local"  username
+#
+[influx local]
+server=127.0.0.1
+username=aaa
+password=aaa
+username_admin=admin
+password_admin=admin
+""")
     return ok
 
 def get_user_pass():
@@ -118,9 +154,11 @@ def call_cmd(CMD2, database=None, fromdb=None, todb=None, silent=False, format="
         #print(CMD)
     CMDx = shlex.split(CMD)
 
-    with open(LOGFILENAME, "a") as f:
-        f.write(f"\nCMD:\n{CMDx}")
-        f.write(f"\n\n")
+
+    logging.debug(f"\nCMD: {CMDx}")
+    #with open(LOGFILENAME, "a") as f:
+    #    f.write(f"\nCMD:\n{CMDx}")
+    #    f.write(f"\n\n")
 
     result = sp.run(CMDx, capture_output=True, text=True )
     if result.returncode != 0:
@@ -142,7 +180,6 @@ def export_measurement(m1, db1, filepath=None, silent=False):
     """
     per parts
     """
-    import csv
     u, p, s = get_user_pass()
     m = s#get_local_machine()
     offset = 0
@@ -169,6 +206,154 @@ def export_measurement(m1, db1, filepath=None, silent=False):
         offset += limit
     return "ok"
 
+# ======================================================================
+def _format_time_ns(ns):
+    ns = int(ns)
+    sec = ns // 1_000_000_000
+    nrem = ns % 1_000_000_000
+    # RFC3339-like with 9-digit fractional seconds (UTC)
+    dat = dt.datetime.fromtimestamp(sec, tz=dt.timezone.utc).strftime("%Y-%m-%dT%H:%M:%S")
+    return f"{dat}.{nrem:09d}Z"
+
+def export_measurement2(m1, db1, filepath=None,
+#                              host='localhost', port=8086,
+#                              username=None, password=None,
+                        window_seconds=24 * 3600,   # time window per query
+                        silent=True,
+                        rows_per_file=1_000_000):
+#                        chunk_size=100000):
+    username, password, host = get_user_pass() #
+    port = 8086
+    client = InfluxDBClient(host=host, port=port, username=username, password=password, database=db1)
+
+    # find time range (ns)
+    q_first = f'SELECT * FROM "{m1}" ORDER BY time ASC LIMIT 1'
+    q_last  = f'SELECT * FROM "{m1}" ORDER BY time DESC LIMIT 1'
+    r1 = client.query(q_first, epoch='ns')
+    r2 = client.query(q_last, epoch='ns')
+    try:
+        first = int(next(r1.get_points(measurement=m1))['time'])
+        last  = int(next(r2.get_points(measurement=m1))['time'])
+    except StopIteration:
+        if not silent:
+            print("measurement empty")
+        client.close()
+        return 0, []
+
+
+    # Build stable header: time + tag keys + field keys
+    tag_keys_res = client.query(f'SHOW TAG KEYS FROM "{m1}"')
+    field_keys_res = client.query(f'SHOW FIELD KEYS FROM "{m1}"')
+    tags = [p['tagKey'] for p in tag_keys_res.get_points()] if tag_keys_res else []
+    fields = [p['fieldKey'] for p in field_keys_res.get_points()] if field_keys_res else []
+    header = ['time'] + tags + fields
+
+    offset = 0
+    limit = 1_000_000
+    now = dt.datetime.now().strftime("%Y%m%d_%H%M")
+    pg = 0
+    hostname = socket.gethostname()
+    #
+    outfilename = f"export_{hostname}_{db1}_{m1}_{now}_{pg:02}.csv"
+    if filepath is not None:
+        outfilename = filepath
+
+
+
+    # output filename handling
+    hostname = socket.gethostname()
+    base_ts = dt.datetime.now().strftime("%Y%m%d_%H%M")
+    page = 0
+    out_files = []
+    rows = 0
+    total = 0
+    fobj = None
+    writer = None
+
+    def open_new_file():
+        nonlocal page, fobj, writer, rows
+        if fobj:
+            fobj.close()
+        page += 1
+        if filepath:
+            fname = filepath
+        else:
+            fname = f"export_{hostname}_{db}_{m1}_{base_ts}_{page:02}.csv"
+        # ensure directory exists if filepath includes dirs
+        os.makedirs(os.path.dirname(fname) or '.', exist_ok=True)
+        fobj = open(fname, 'w', newline='')
+        w = csv.DictWriter(fobj, fieldnames=header)
+        w.writeheader()
+        writer = w
+        rows = 0
+        if not filepath:
+            out_files.append(fname)
+        else:
+            # for filepath case record single file once
+            if not out_files:
+                out_files.append(fname)
+
+    # begin streaming windows
+    window_ns = int(window_seconds * 1_000_000_000)
+    start = first
+    open_new_file()
+
+    while start <= last:
+        end = start + window_ns
+        q = f'SELECT * FROM "{m1}" WHERE time >= {start} AND time < {end} ORDER BY time ASC'
+        res = client.query(q, epoch='ns')  # no chunked=True
+        pts = res.get_points(measurement=m1)
+        for p in pts:
+            t_ns = int(p.pop('time'))
+            row = {'time': _format_time_ns(t_ns)}
+            # tags first, then fields to match header
+            for k in tags + fields:
+                v = p.get(k, '')
+                row[k] = '' if v is None else v
+            writer.writerow(row)
+            rows += 1
+            total += 1
+            # rotate file if needed (only when filepath is None)
+            if not filepath and rows >= rows_per_file:
+                open_new_file()
+        start = end
+
+    if fobj:
+        fobj.close()
+    client.close()
+    if not silent:
+        print(f"exported {total} points into {len(out_files)} file(s)")
+    return total, out_files
+
+    # # query = f'SELECT * FROM "{m1}" ORDER BY time ASC'  # QUERY *********************
+    # # ok = False
+    # # try:
+    # #     result = client.query(query, chunked=True, chunk_size=chunk_size, epoch='ns')
+    # #     ok = True
+    # # except:
+    # #     pass
+    # # if not ok:return 0
+
+
+    # points_iter = result.get_points(measurement=m1)
+
+    # with open(outfilename, 'w', newline='') as f:
+    #     writer = csv.DictWriter(f, fieldnames=header)
+    #     writer.writeheader()
+    #     pts = result.get_points(measurement=m1)
+    #     for p in pts:
+    #         # ensure time present and formatted with ns precision
+    #         t_ns = int(p.pop('time'))
+    #         row = {'time': _format_time_ns(t_ns)}
+    #         # write tags/fields in header order; missing -> empty string
+    #         for k in tags + fields:
+    #             # convert None to empty, keep numeric types as-is
+    #             v = p.get(k, '')
+    #             row[k] = '' if v is None else v
+    #         writer.writerow(row)
+
+    # client.close()
+    # return True
 
 # ================================================================================
 # backup ALL                  separate call      backup ONE  database too
@@ -185,12 +370,13 @@ def backup_portable(PATH, database=None):
     #CMD = f"influxd backup -skip-errors -portable  {os.path.expanduser(PATH)}/ "
     CMD = f"influxd backup  -portable {dbstring}   {os.path.expanduser(PATH)}/ "
     CMDx = shlex.split(CMD)
-    with open(LOGFILENAME, "a") as f:
-        f.write("\n\n")
-        f.write(f"\n* BACKING UP {database} to  {PATH}\n")
-        f.write(CMD + "\n")
-        f.write("CMDx\n")
-        f.write(f"{CMDx}\n")
+    logging.debug(f"\n* BACKING UP {database} to  {PATH}\n")
+    # with open(LOGFILENAME, "a") as f:
+    #     f.write("\n\n")
+    #     f.write(f"\n* BACKING UP {database} to  {PATH}\n")
+    #     f.write(CMD + "\n")
+    #     f.write("CMDx\n")
+    #     f.write(f"{CMDx}\n")
     sp.run(CMD, shell=True)
     #---------------------------------------------------
 
@@ -204,40 +390,46 @@ def restore_portable_get_oldname(PATH, database=None):
     """
     dbstring = ""
     if database is  None:
-        with open(LOGFILENAME, "a") as f:
-            f.write(f"\nIMPOSSIBLE TO RESTORE FULL INFLUX\ndo database after database\n\n")
+        logging.debug(f"\nIMPOSSIBLE TO RESTORE FULL INFLUX\ndo database after database\n\n")
+        #with open(LOGFILENAME, "a") as f:
+        #    f.write(f"\nIMPOSSIBLE TO RESTORE FULL INFLUX\ndo database after database\n\n")
         return None
     manifest_file = glob.glob(f"{PATH}/*manifest")
     if len(manifest_file) != 1:
-        with open(LOGFILENAME, "a") as f:
-            f.write(f"\nmanifest files != 1 {manifest_file} \n\n")
+        logging.debug(f"\nmanifest files != 1 {manifest_file} \n\n")
+        #with open(LOGFILENAME, "a") as f:
+        #    f.write(f"\nmanifest files != 1 {manifest_file} \n\n")
         return None
 
     # --------------------------------------------------------
     manidbs = None
-    with open(LOGFILENAME, "a") as f:
-        f.write(f"\nmanifest_file:    {manifest_file} \n\n")
+    logging(f"\nmanifest_file:    {manifest_file} \n\n")
+    #with open(LOGFILENAME, "a") as f:
+    #    f.write(f"\nmanifest_file:    {manifest_file} \n\n")
 
     with open(manifest_file[0], "r") as f:
         manifest = json.load(f)
 
-
-        with open(LOGFILENAME, "a") as f:
-            f.write(f"\nmanifest_json[files]:    {manifest['files']} \n\n")
+        logging.debug(f"\nmanifest_json[files]:    {manifest['files']} \n\n")
+        #with open(LOGFILENAME, "a") as f:
+        #    f.write(f"\nmanifest_json[files]:    {manifest['files']} \n\n")
 
         if manifest['files'] is None:
             return None
 
         manidbs = { entry['database'] for entry in manifest['files'] }
 
-        with open(LOGFILENAME, "a") as f:
-            f.write(f"\nmanifest dbs type:    {manidbs}  {type(manidbs)}  ==set\n\n")
-            f.write(f"\nmanifest dbs len :    {manidbs}  {len(manidbs)}    ==1\n\n")
-            ##f.write(f"\nmanifest:    {manidbs}  {manidbs[0]}    ==1\n\n")
+        logging.debug(f"\nmanifest dbs type:    {manidbs}  {type(manidbs)}  ==set\n\n")
+        logging.debug(f"\nmanifest dbs len :    {manidbs}  {len(manidbs)}    ==1\n\n")
+        #with open(LOGFILENAME, "a") as f:
+        #    f.write(f"\nmanifest dbs type:    {manidbs}  {type(manidbs)}  ==set\n\n")
+        #    f.write(f"\nmanifest dbs len :    {manidbs}  {len(manidbs)}    ==1\n\n")
+        #    ##f.write(f"\nmanifest:    {manidbs}  {manidbs[0]}    ==1\n\n")
 
         if len(manidbs) > 1:
-            with open(LOGFILENAME, "a") as f:
-                f.write(f"\n problem - manifest file databases != 1 {manidbs} \n\n")
+            logging.debug(f"\n problem - manifest file databases != 1 {manidbs} \n\n")
+            #with open(LOGFILENAME, "a") as f:
+            #    f.write(f"\n problem - manifest file databases != 1 {manidbs} \n\n")
             return None
     original_database = next(iter(manidbs))
     return original_database
@@ -249,8 +441,9 @@ def restore_portable(PATH, database=None):
     """
     dbstring = ""
     if database is  None:
-        with open(LOGFILENAME, "a") as f:
-            f.write(f"\nIMPOSSIBLE TO RESTORE FULL INFLUX\ndo database after database\n\n")
+        logging.debug(f"\nIMPOSSIBLE TO RESTORE FULL INFLUX\ndo database after database\n\n")
+        #with open(LOGFILENAME, "a") as f:
+        #    f.write(f"\nIMPOSSIBLE TO RESTORE FULL INFLUX\ndo database after database\n\n")
         return
 
     original_database = restore_portable_get_oldname(PATH, database)
@@ -261,14 +454,18 @@ def restore_portable(PATH, database=None):
     #CMD = f"influxd backup -skip-errors -portable  {os.path.expanduser(PATH)}/ "
     CMD = f"influxd restore  -portable  {dbstring}   {os.path.expanduser(PATH)}/ "
     CMDx = shlex.split(CMD)
-    with open(LOGFILENAME, "a") as f:
-        f.write(f"\n* RESTORING {database} from  {PATH}\n")
-        f.write(CMD + "\n")
-        f.write("CMDx:  ")
-        f.write(f"{CMDx}\n")
+    logging.debug(f"\n* RESTORING {database} from  {PATH}\n")
+    logging.debug(f"\n*  {CMD} \n")
+    logging.debug(f"\n*  {CMDx} \n")
+    #with open(LOGFILENAME, "a") as f:
+    #    f.write(f"\n* RESTORING {database} from  {PATH}\n")
+    #    f.write(CMD + "\n")
+    #    f.write("CMDx:  ")
+    #    f.write(f"{CMDx}\n")
     sp.run(CMD, shell=True)
-    with open(LOGFILENAME, "a") as f:
-        f.write(f"\n... should be restored now\n")
+    logging.debug(f"\n... should be restored now\n")
+    #with open(LOGFILENAME, "a") as f:
+    #    f.write(f"\n... should be restored now\n")
     #---------------------------------------------------
 
 
@@ -363,13 +560,103 @@ def show_measurements(database):
     print(res)
     return res
 
-def copy_measurement(m1, db1, db2, silent=False, new_measurement_name=None):
+
+def copy_measurement2(m1, db1, db2, silent=False, new_measurement=None,
+#                     host='localhost', port=8086, username=None, password=None,
+#                      chunk_size=1000,
+                      window_seconds=24 * 3600,  # 1 hour windows, tune as needed
+                      batch_size=5000):
+    """
+    copy with client . If new_measurement provided, rename on write.
+    Returns number of points copied.
+    """
+    username, password, host = get_user_pass() #
+    port = 8086
+
+    target = new_measurement or m1
+    src = InfluxDBClient(host=host, port=port, username=username, password=password, database=db1)
+    dst = InfluxDBClient(host=host, port=port, username=username, password=password, database=db2)
+
+
+    if not silent:
+        print(fg.blue, f"copy measurement {m1} from {db1} to {db2}", fg.default)
+
+    # get tag keys (to separate tags vs fields)
+    tag_keys_res = src.query(f'SHOW TAG KEYS FROM "{m1}"')
+    tags_list = [p['tagKey'] for p in tag_keys_res.get_points()] if tag_keys_res else []
+
+    total = 0
+    batch = []
+
+    # stream points as nanoseconds so we preserve sub-second precision
+    # get earliest and latest timestamps (ns)
+    q_first = f'SELECT * FROM "{m1}" ORDER BY time ASC LIMIT 1'
+    q_last  = f'SELECT * FROM "{m1}" ORDER BY time DESC LIMIT 1'
+    r1 = src.query(q_first, epoch='ns')
+    r2 = src.query(q_last, epoch='ns')
+
+    try:
+        t0 = int(next(r1.get_points(measurement=m1))['time'])
+        t_end = int(next(r2.get_points(measurement=m1))['time'])
+    except StopIteration:
+        if not silent:
+            print("measurement empty")
+        src.close(); dst.close(); return 0
+
+
+    window_ns = int(window_seconds * 1_000_000_000)
+    total = 0
+    batch = []
+
+
+    start = t0
+    while start <= t_end:
+        end = start + window_ns
+        q = f'SELECT * FROM "{m1}" WHERE time >= {start} AND time < {end} ORDER BY time ASC'
+        # no chunked=True here — avoid the msgpack streaming path
+        res = src.query(q, epoch='ns')
+        pts = res.get_points(measurement=m1)
+        for p in pts:
+            t_ns = int(p.pop('time'))
+            # separate tags vs fields is optional; here we treat everything not None as field.
+            # If you need to preserve tags specifically, fetch SHOW TAG KEYS and split.
+            fields = {k: v for k, v in p.items() if v is not None}
+            if not fields:
+                continue
+            point = {
+                "measurement": target,
+                "time": t_ns,
+                "fields": fields
+            }
+            batch.append(point)
+            if len(batch) >= batch_size:
+                dst.write_points(batch, time_precision='n', database=db2)
+                total += len(batch)
+                batch = []
+        start = end
+
+    if batch:
+        dst.write_points(batch, time_precision='n', database=db2)
+        total += len(batch)
+
+    src.close(); dst.close()
+
+    if not silent:
+        print(f"copied {total} points")
+    return total
+
+
+
+def copy_measurement(m1, db1, db2, silent=False, new_measurement=None):
+    """
+    new_measurement is for rename...
+    """
     if not silent:
         print(fg.blue, f"copy measurement {m1} from {db1} to {db2}", fg.default)
     CMD = f"SELECT * INTO {db2}..{m1} FROM {db1}..{m1} group by *"
-    if new_measurement_name is not None:
+    if new_measurement is not None:
         # THE CORRECT ORDER!!!
-        CMD = f"SELECT * INTO {db2}..{new_measurement_name} FROM {db1}..{m1} group by *"
+        CMD = f"SELECT * INTO {db2}..{new_measurement} FROM {db1}..{m1} group by *"
     if not silent:
         print(CMD)
     res = CMD
